/*
 * Copyright 2010 IHE International (http://www.ihe.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.ihe.gazelle.proxy.model.message;

import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.sql.Timestamp;
import java.util.Arrays;
import java.util.Iterator;
import java.util.Set;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Lob;
import javax.persistence.Transient;

import net.ihe.gazelle.proxy.dicom.TxtInputHandler;
import net.ihe.gazelle.proxy.netty.ChannelType;
import net.ihe.gazelle.proxy.netty.channel.ProxySide;
import net.ihe.gazelle.proxy.util.Preferences;

import org.apache.commons.httpclient.methods.multipart.FilePartSource;
import org.apache.commons.httpclient.methods.multipart.PartSource;
import org.apache.commons.lang.StringEscapeUtils;
import org.dcm4che2.data.BasicDicomObject;
import org.jboss.seam.annotations.Name;

import com.pixelmed.dicom.Attribute;
import com.pixelmed.dicom.AttributeList;
import com.pixelmed.dicom.AttributeTag;
import com.pixelmed.dicom.DicomDictionary;
import com.pixelmed.dicom.DicomException;
import com.pixelmed.dicom.DicomInputStream;
import com.pixelmed.dicom.SOPClassDescriptions;
import com.pixelmed.network.MessageServiceElementCommand;

@Entity
@Name("dicomMessage")
public class DicomMessage extends AbstractMessage implements java.io.Serializable {

	private static final DicomDictionary DICOM_DICTIONARY = new DicomDictionary();

	/** Serial ID version of this object */
	private static final long serialVersionUID = 54153409123922L;

	@Column(name = "dicom_filecommandset")
	@Basic(fetch = FetchType.LAZY)
	private byte[] fileCommandSet;

	@Column(name = "dicom_filetransfertsyntax")
	@Lob
	private String fileTransfertSyntax;

	@Lob
	private String fileDump;

	public ChannelType getChannelType() {
		return ChannelType.DICOM;
	}

	// Constructors

	public byte[] getFileCommandSet() {
		return fileCommandSet;
	}

	public void setFileCommandSet(byte[] fileCommandSet) {
		this.fileCommandSet = fileCommandSet;
	}

	public String getFileTransfertSyntax() {
		return fileTransfertSyntax;
	}

	public void setFileTransfertSyntax(String fileTransfertSyntax) {
		this.fileTransfertSyntax = fileTransfertSyntax;
	}

	public String getAttributesForCommandSetString() {
		return dumpFileCommandSet(false);
	}

	private String dumpFileCommandSet(boolean compact) {
		if (getFileCommandSet() != null) {
			AttributeList attributeList = new AttributeList();

			try {
				attributeList.read(new DicomInputStream(new ByteArrayInputStream(getFileCommandSet())));
				Set<AttributeTag> keys = attributeList.keySet();
				Iterator<AttributeTag> iterator = keys.iterator();

				StringWriter sw = new StringWriter();
				BufferedWriter bw = new BufferedWriter(sw);

				while (iterator.hasNext()) {
					AttributeTag tag = iterator.next();
					Attribute attributeValue = attributeList.get(tag);

					if (!compact) {
						bw.append("<p><span style=\"font-family: monospace;\">");
						bw.append(StringEscapeUtils.escapeHtml(attributeValue.toString(DICOM_DICTIONARY)));
						bw.append("</span></p>");
						bw.newLine();
					}

					// affected sop class
					if (tag.equals(DICOM_DICTIONARY.getTagFromName("AffectedSOPClassUID"))) {
						try {
							String fieldValue = attributeValue.getDelimitedStringValuesOrEmptyString();
							String affectedSopClassUID = SOPClassDescriptions.getDescriptionFromUID(fieldValue);
							if (affectedSopClassUID == null) {
								affectedSopClassUID = fieldValue;
							}
							if (!compact) {
								bw.append("<p>Affected SOP class : <span style=\"font-family: monospace;\">");
								bw.append(StringEscapeUtils.escapeHtml(affectedSopClassUID));
								bw.append("</span></p>");
								bw.newLine();
							} else {
								bw.append("Aff. <span style=\"font-family: monospace;\">");
								bw.append(StringEscapeUtils.escapeHtml(affectedSopClassUID));
								bw.append("</span><br />");
							}
						} catch (Exception e) {
							e.printStackTrace();
						}
					}

					// Command field
					if (tag.equals(DICOM_DICTIONARY.getTagFromName("CommandField"))) {
						try {
							if (!compact) {
								bw.append("<p>Command fields : <span style=\"font-family: monospace;\">");
							} else {
								bw.append("Cmd. <span style=\"font-family: monospace;\">");
							}
							int[] integerValues = attributeValue.getIntegerValues();
							for (Integer fieldValue : integerValues) {
								String commandFieldTmp = MessageServiceElementCommand.toString(fieldValue);
								bw.append(StringEscapeUtils.escapeHtml(commandFieldTmp));
								bw.append(" ");
							}
							if (!compact) {
								bw.append("</span></p>");
								bw.newLine();
							} else {
								bw.append("</span><br />");
							}
						} catch (Exception e) {
							e.printStackTrace();
						}
					}

					// requested sop class
					if (tag.equals(DICOM_DICTIONARY.getTagFromName("RequestedSOPClassUID"))) {
						try {
							String fieldValue = attributeValue.getDelimitedStringValuesOrEmptyString();
							String requestedSopClassUID = SOPClassDescriptions.getDescriptionFromUID(fieldValue);
							if (requestedSopClassUID == null) {
								requestedSopClassUID = fieldValue;
							}
							if (!compact) {
								bw.append("<p>Requested SOP class : <span style=\"font-family: monospace;\">");
								bw.append(StringEscapeUtils.escapeHtml(requestedSopClassUID));
								bw.append("</span></p>");
								bw.newLine();
							} else {
								bw.append("Req. <span style=\"font-family: monospace;\">");
								bw.append(StringEscapeUtils.escapeHtml(requestedSopClassUID));
								bw.append("</span><br />");
							}
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
					if (!compact) {
						if (iterator.hasNext()) {
							bw.append("<hr />");
						}
					}
				}
				bw.close();
				return sw.toString();

			} catch (IOException e) {
				e.printStackTrace();
			} catch (DicomException e) {
				e.printStackTrace();
			}
		}
		return "";
	}

	public String getFileDump() {
		return fileDump;
	}

	public void setFileDump(String fileDump) {
		this.fileDump = fileDump;
	}

	public DicomMessage() {
		super();
	}

	public DicomMessage(Timestamp timestamp, String fromIP, Integer localPort, Integer proxyPort, String toIP,
			Integer remotePort, ProxySide proxySide) {
		super(timestamp, fromIP, localPort, proxyPort, toIP, remotePort, proxySide);
	}

	public static String humanReadableByteCount(long bytes, boolean si) {
		int unit = si ? 1000 : 1024;
		if (bytes < unit)
			return bytes + " B";
		int exp = (int) (Math.log(bytes) / Math.log(unit));
		String pre = (si ? "kMGTPE" : "KMGTPE").charAt(exp - 1) + (si ? "" : "i");
		return String.format("%.1f %sB", bytes / Math.pow(unit, exp), pre);
	}

	@Override
	public String getInfoGUI() {
		return dumpFileCommandSet(true);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result + Arrays.hashCode(fileCommandSet);
		result = prime * result + ((fileTransfertSyntax == null) ? 0 : fileTransfertSyntax.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!super.equals(obj))
			return false;
		if (getClass() != obj.getClass())
			return false;
		DicomMessage other = (DicomMessage) obj;
		if (!Arrays.equals(fileCommandSet, other.fileCommandSet))
			return false;
		if (fileTransfertSyntax == null) {
			if (other.fileTransfertSyntax != null)
				return false;
		} else if (!fileTransfertSyntax.equals(other.fileTransfertSyntax))
			return false;
		return true;
	}

	private File getFile() {
		File storageFolder = new File(Preferences.getProperty("storage.dicom"));
		return new File(storageFolder, getMessageReceivedAsString());
	}

	@Override
	public int getMessageLength() {
		try {
			return (int) getFile().length();
		} catch (Throwable e) {
			return 0;
		}
	}

	@Override
	public InputStream getMessageReceivedStream() {
		try {
			return new FileInputStream(getFile());
		} catch (FileNotFoundException e) {
			return null;
		}
	}

	@Override
	public PartSource getPartSource(String messageType) {
		try {
			return new FilePartSource(messageType, getFile());
		} catch (FileNotFoundException e) {
			return null;
		}
	}

	public void dumpDicomFile() {
		org.dcm4che2.io.DicomInputStream dis = null;
		try {
			dis = new org.dcm4che2.io.DicomInputStream(getFile());
			StringWriter sw = new StringWriter();
			BufferedWriter writer = new BufferedWriter(sw);
			dis.setHandler(new TxtInputHandler(writer));
			dis.readDicomObject(new BasicDicomObject(), -1);
			writer.close();
			fileDump = sw.toString();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (dis != null) {
				try {
					dis.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

}
