package net.ihe.gazelle.proxy.util;

import static org.jboss.seam.annotations.Install.APPLICATION;

import java.security.Principal;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;

import javax.faces.context.FacesContext;
import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.commons.lang.StringUtils;
import org.jasig.cas.client.authentication.AttributePrincipal;
import org.jasig.cas.client.util.AbstractCasFilter;
import org.jasig.cas.client.validation.Assertion;
import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Install;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.Startup;
import org.jboss.seam.annotations.intercept.BypassInterceptors;
import org.jboss.seam.contexts.Contexts;
import org.jboss.seam.core.Events;
import org.jboss.seam.faces.FacesManager;
import org.jboss.seam.log.LogProvider;
import org.jboss.seam.log.Logging;
import org.jboss.seam.security.Identity;

@Name("org.jboss.seam.security.identity")
@Scope(ScopeType.SESSION)
@Install(precedence = APPLICATION)
@BypassInterceptors
@Startup
public class SSOIdentity extends Identity {

	private static final long serialVersionUID = -631532323964539777L;

	public static final String AUTHENTICATED_USER = "org.jboss.seam.security.management.authenticatedUser";
	public static final String EVENT_USER_AUTHENTICATED = "org.jboss.seam.security.management.userAuthenticated";
	private static final String SILENT_LOGIN = "org.jboss.seam.security.silentLogin";

	private static final LogProvider log = Logging.getLogProvider(SSOIdentity.class);

	@Override
	public String login() {
		log.info("Starting authentication");

		try {

			if (super.isLoggedIn()) {
				// If authentication has already occurred during this request
				// via a silent login, and login() is explicitly called then we
				// still want to raise the LOGIN_SUCCESSFUL event, and then
				// return.
				if (Contexts.isEventContextActive() && Contexts.getEventContext().isSet(SILENT_LOGIN)) {
					if (Events.exists())
						Events.instance().raiseEvent(EVENT_LOGIN_SUCCESSFUL);
					return "loggedIn";
				}

				if (Events.exists())
					Events.instance().raiseEvent(EVENT_ALREADY_LOGGED_IN);
				return "loggedIn";
			}

			preAuthenticate();
			AttributePrincipal attributePrincipal = null;
			Principal casPrincipal = getCASPrincipal();
			if (casPrincipal instanceof AttributePrincipal) {
				attributePrincipal = (AttributePrincipal) casPrincipal;
			}

			if (attributePrincipal == null) {
				return super.login();
			} else if (casPrincipal.getName() != null) {
				preAuthenticate();

				String username = casPrincipal.getName();

				log.info("Found CAS principal for " + username + ": authenticated");
				Map attributes = attributePrincipal.getAttributes();
				Set<Map.Entry> entrySet = attributes.entrySet();
				for (Map.Entry object : entrySet) {
					log.info("       " + object.getKey() + " = " + object.getValue());
				}

				acceptExternallyAuthenticatedPrincipal(casPrincipal);
				if (attributes != null && attributes.get("role_name") != null) {
					String roles = (String) attributes.get("role_name");
					roles = roles.substring(1, roles.length() - 1);
					StringTokenizer st = new StringTokenizer(roles, ",");
					String role;
					while (st.hasMoreElements()) {
						role = (String) st.nextElement();
						role = StringUtils.trimToNull(role);
						if (role != null) {
							addRole(role);
						}
					}
				}
				getCredentials().setUsername(username);
				postAuthenticate();
				return "loggedIn";

			}

		} catch (Throwable e) {
			unAuthenticate();
			throw new RuntimeException(e);
		}

		return null;
	}

	private Principal getCASPrincipal() {
		final HttpServletRequest request = (HttpServletRequest) FacesContext.getCurrentInstance().getExternalContext()
				.getRequest();
		final HttpSession session = request.getSession(false);
		final Assertion assertion = (Assertion) (session == null ? request
				.getAttribute(AbstractCasFilter.CONST_CAS_ASSERTION) : session
				.getAttribute(AbstractCasFilter.CONST_CAS_ASSERTION));
		return assertion == null ? null : assertion.getPrincipal();
	}

	@Override
	public boolean isLoggedIn() {
		if (!super.isLoggedIn()) {
			if (getCASPrincipal() != null) {
				login();
			}
		}
		return super.isLoggedIn();
	}

	@Override
	public void logout() {
		Object request = FacesContext.getCurrentInstance().getExternalContext().getRequest();
		if (request instanceof ServletRequest) {
			ServletRequest servletRequest = (ServletRequest) request;
			servletRequest.setAttribute(AbstractCasFilter.CONST_CAS_ASSERTION, null);
		}
		Object session = FacesContext.getCurrentInstance().getExternalContext().getSession(false);
		if (session instanceof HttpSession) {
			HttpSession httpSession = (HttpSession) session;
			httpSession.setAttribute(AbstractCasFilter.CONST_CAS_ASSERTION, null);
		}
		super.logout();
		FacesManager.instance().redirectToExternalURL("https://gazelle.ihe.net/logout");
	}

}
